@@ -60,14 +60,17 @@ class QuantizedGELU(Calibrable, keras.layers.Layer):
     def __init__(self, *args, name="i_gelu", quant_config={}, **kwargs):
         super().__init__(*args, name=name, **kwargs)
         self.quant_config = quant_config
-        self.bitwidth = self.quant_config.get(
+        self.buffer_bitwidth = self.quant_config.get(
             "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
-        assert self.bitwidth >= 0, "The buffer_bitwidth must be a non-negative integer."
+        assert self.buffer_bitwidth >= 0, "The buffer_bitwidth must be a non-negative integer."
 
         # All constants are in the range |x| < 2. We only need 2bits for the int part
         # and remove the sign bit.
         value_bits = self.quant_config.get("weight_bitwidth", 10) - 1
         frac_bits = value_bits - 2
+        # Finally, we make an implicit promote
+        assert self.buffer_bitwidth > value_bits, "buffer_bw must be higher than weight_bw"
+        value_bits = self.buffer_bitwidth
         # Coefficients of the degree 2 polynomial, used to approximate erf(x/sqrt(2))
         self._a_int = FixedPoint.quantize(-0.2888, frac_bits, value_bits)
         self._b_int = FixedPoint.quantize(-1.769, frac_bits, value_bits)
@@ -103,7 +106,15 @@ class QuantizedGELU(Calibrable, keras.layers.Layer):
         Note:
             ``S`` denotes the quantization scale.
         """
-        return (q_in + q_b)**2 * q_a + q_c
+        # Align intermediate inputs before adding them
+        q_in, _ = q_in.align(q_b)
+        q_b, _ = q_b.align(q_in)
+        q_inter = (q_in + q_b)**2 * q_a
+
+        # Align intermediate results before adding them
+        q_inter, _ = q_inter.align(q_c)
+        q_c, _ = q_c.align(q_inter)
+        return  q_inter + q_c
 
     def i_erf(self, q_in):
         """Approximate the error function with a degree two polynomial.
@@ -155,7 +166,7 @@ class QuantizedGELU(Calibrable, keras.layers.Layer):
                              Receives {type(inputs)} inputs.")
 
         # 1. Increase the bitwidth to desired value
-        inputs = inputs.promote(self.bitwidth)
+        inputs = inputs.promote(self.buffer_bitwidth)
         # 2. Approximate the GELU function with i-GELU
         outputs = self.i_gelu(inputs)
 
