@@ -141,6 +141,10 @@ class QuantizedConv2D(Calibrable, Conv2D):
 
         if self.use_bias:
             bias = self.bias_quantizer(self.bias, training)
+
+            # Align intermediate outputs and biases before adding them
+            outputs, _ = outputs.align(bias)
+            bias, _ = bias.promote(self.buffer_bitwidth).align(outputs)
             outputs = tf.add(outputs, bias)
 
         if self.out_quantizer is not None:
