@@ -75,7 +75,12 @@ class QuantizedDepthwiseConv2D(Calibrable, DepthwiseConv2D):
             data_format=self.data_format)
 
         if self.use_bias:
-            outputs = tf.add(outputs, self.bias_quantizer(self.bias, training))
+            bias = self.bias_quantizer(self.bias, training)
+
+            # Align intermediate outputs and biases before adding them
+            outputs, _ = outputs.align(bias)
+            bias, _ = bias.promote(self.buffer_bitwidth).align(outputs)
+            outputs = tf.add(outputs, bias)
 
         if self.out_quantizer is not None:
             outputs = self.out_quantizer(outputs)
