@@ -14,6 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 # ******************************************************************************
+import keras
 import tensorflow as tf
 
 from ..tensors import FixedPoint
@@ -23,7 +24,7 @@ __all__ = ["QDropout"]
 
 
 @tf.keras.utils.register_keras_serializable()
-class QDropout(tf.keras.layers.Dropout):
+class QDropout(keras.layers.Dropout):
 
     def call(self, inputs, training=None):
         # raise an error if the inputs are not FixedPoint
