@@ -21,8 +21,7 @@ import tensorflow as tf
 from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
 
 
-__all__ = ["Add", "QuantizedAdd", "ExtractToken",
-           "QuantizedExtractToken", "deserialize_quant_object"]
+__all__ = ["Add", "QuantizedAdd", "deserialize_quant_object"]
 
 
 def has_shift_info():
@@ -238,6 +237,11 @@ class QuantizedAdd(Calibrable, Add):
 
         # Promote a before performing the addition
         a = a.promote(self.buf_bits)
+        b = b.promote(self.buf_bits)
+
+        # Align intermediate inputs before adding them
+        a, shift_ab = a.align(b)
+        b, shift_ba = b.align(a)
 
         outputs = tf.add(a, b)
         if self.average:
@@ -245,8 +249,8 @@ class QuantizedAdd(Calibrable, Add):
 
         # Compute shifts
         if self.has_shift_info:
-            self.shift_add_a(outputs.frac_bits - a.frac_bits)
-            self.shift_add_b(outputs.frac_bits - b.frac_bits)
+            self.shift_add_a(shift_ab)
+            self.shift_add_b(shift_ba)
 
         if self.out_quantizer is not None:
             internal_out_frac_bits = outputs.frac_bits
@@ -262,38 +266,3 @@ class QuantizedAdd(Calibrable, Add):
         config = super().get_config()
         config["quant_config"] = self.quant_config
         return config
-
-
-@tf.keras.utils.register_keras_serializable()
-class ExtractToken(keras.layers.Layer):
-    """ Wrapper class of `tf.gather` operation that allows to extract a Token.
-
-    Args:
-        token (int): the indice of the token to extract.
-
-    """
-
-    def __init__(self, *args, token, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.token = token
-
-    def call(self, inputs, training=False):
-        return tf.gather(inputs, self.token, axis=1)
-
-    def get_config(self):
-        config = super().get_config()
-        config.update({"token": self.token})
-        return config
-
-
-@tf.keras.utils.register_keras_serializable()
-class QuantizedExtractToken(ExtractToken):
-    """ Quantized version of the ExtractToken layer. Accepts only FixedPoint inputs.
-    """
-
-    def call(self, inputs, training=False):
-        if not isinstance(inputs, FixedPoint):
-            # If the inputs is not a FixedPoint, raise an error
-            raise TypeError(f"QuantizedExtractToken only accepts FixedPoint\
-                              inputs. Receives {type(inputs)} inputs.")
-        return tf.gather(inputs, self.token, axis=1)
