@@ -158,7 +158,9 @@ class QuantizedLayerNormalization(Calibrable, LayerMadNormalization):
         sum_x = tf.math.reduce_sum(inputs, self.axis, keepdims=True)
         mean_x = sum_x // nb_channels
 
-        # Compute the Absolute Deviation (MAD)
+        # Compute the Absolute Deviation (MAD), aligned inputs and mean before
+        inputs, _ = inputs.align(mean_x)
+        mean_x, _ = mean_x.align(inputs)
         x_shifted = inputs - mean_x
         mad_sum = tf.math.reduce_sum(
             x_shifted.abs(), self.axis, keepdims=True, name="mad")
@@ -178,7 +180,12 @@ class QuantizedLayerNormalization(Calibrable, LayerMadNormalization):
         # by the left term. In the multiplication below, we want to keep the higher
         # frac_bits value, which is owned by "mad_inv" variable. Hence "mad_inv" should
         # be the left term.
-        outputs = mad_inv * gamma * x_shifted + beta
+        outputs = mad_inv * gamma * x_shifted
+
+        # Before adding the bias, we need aligned results
+        outputs, _ = outputs.align(beta)
+        beta, _ = beta.promote(self.buffer_bitwidth).align(outputs)
+        outputs = tf.add(outputs, beta)
         outputs.values.set_shape(inputs.shape)
 
         # If no quantizer output is provided return the FixedPoint output tensor directly
