@@ -212,15 +212,21 @@ class Quantizer(Calibrable, Layer):
             :obj:`tensorflow.Tensor`: the quantized tensor.
         """
         if isinstance(inputs, QTensor):
-            inputs = inputs.to_float()
+            if inputs.value_bits <= self.value_bits:
+                raise ValueError(f"Quantizing a {inputs.value_bits}-bit QTensor to {self.value_bits}-bit"
+                    " is pointless: use a promotion instead.")
         if self.calibration:
+            if isinstance(inputs, QTensor):
+                new_inputs = inputs.to_float()
+            else:
+                new_inputs = inputs
             # Retrieve information from the inputs and update the weights
-            input_batch_size = tf.shape(inputs)[0]
+            input_batch_size = tf.shape(new_inputs)[0]
             if tf.reduce_all(tf.math.equal(self.max_value, tf.constant(1.))):
                 momentum = tf.constant(-1.)
             else:
                 momentum = tf.convert_to_tensor(self.momentum)
-            self._update_weights(inputs, self.max_value,
+            self._update_weights(new_inputs, self.max_value,
                                  self.axis, input_batch_size, momentum)
         return self.quantize(inputs)
 
@@ -264,8 +270,8 @@ class UniformQuantizer(Quantizer):
         Returns:
             a QFloat
         """
-        if not self.signed:
-            x = tf.math.maximum(x, 0)
+        if isinstance(x, QTensor):
+            x = x.to_float()
         float_max = tf.stop_gradient(self.max_value)
         return QFloat.quantize(x, float_max, self.value_bits, self.scale_bits)
 
@@ -311,9 +317,11 @@ class FixedPointQuantizer(Quantizer):
         Returns:
             a FixedPoint
         """
+        if isinstance(x, QFloat):
+            x = x.to_float()
         frac_bits = tf.stop_gradient(self.frac_bits)
-        if not self.signed:
-            x = tf.math.maximum(x, 0)
+        if isinstance(x, FixedPoint):
+            return x.downscale(frac_bits, self.value_bits)
         return FixedPoint.quantize(x, frac_bits, self.value_bits)
 
 
