@@ -20,12 +20,13 @@ import warnings
 
 from keras import Model
 
-from .utils import deep_clone_model
+from .utils import deep_clone_model, insert_layer
 from ..layers import (QuantizedDense, QuantizedLayerNormalization, QuantizedAddPositionEmbs,
                       QuantizedClassToken, QuantizedGELU, QuantizedReLU, QuantizedAdd, QDropout,
                       QuantizedReshape, QuantizedAttention, QuantizedConv2D, QuantizedMaxPool2D,
                       QuantizedGlobalAveragePooling2D, QuantizedExtractToken, Dequantizer,
-                      Quantizer, QuantizedSeparableConv2D, QuantizedFlatten)
+                      Quantizer, QuantizedSeparableConv2D, QuantizedFlatten, QuantizedReciprocal,
+                      FixedPointQuantizer, UniformQuantizer)
 
 # Mapper to match float layer with its quantized version
 CUSTOM_QLAYERS = {
@@ -36,8 +37,51 @@ CUSTOM_QLAYERS = {
     "Attention": QuantizedAttention, 'MaxPooling2D': QuantizedMaxPool2D,
     'GlobalAveragePooling2D': QuantizedGlobalAveragePooling2D,
     "ExtractToken": QuantizedExtractToken, "SeparableConv2D": QuantizedSeparableConv2D,
-    "Flatten": QuantizedFlatten
+    "Flatten": QuantizedFlatten, "Reciprocal": QuantizedReciprocal,
 }
+# List of Quantizer layer's that do not have a float layer representation
+NO_FLOAT_CUSTOM_QLAYERS = [Dequantizer, FixedPointQuantizer, UniformQuantizer]
+
+
+def _handle_not_quantizable_layers(model):
+    """ Checks if the model has not quantizable layers and adds a Dequantizer before.
+
+    Args:
+        model (keras.Model): model to check
+
+    Returns:
+        keras.Model: the updated model
+    """
+    def is_quantizable(layer):
+        layer_class = layer.__class__
+        return (layer_class.__name__ in CUSTOM_QLAYERS or layer_class in CUSTOM_QLAYERS.values()
+                or layer_class in NO_FLOAT_CUSTOM_QLAYERS)
+
+    # Find layers that cannot be quantized
+    for layer in model.layers:
+        if not is_quantizable(layer):
+            # This layer cannot be quantized, check its inbounds
+            inbound = layer.inbound_nodes[0]
+            if not inbound.inbound_layers:
+                # Skip input layers
+                continue
+            elif isinstance(inbound.inbound_layers, list):
+                raise RuntimeError(f"'{layer.name}' is not quantizable and has multiple inbounds "
+                                    "which is not supported.")
+
+            # Check if the layer inbound will be quantized but is not a Dequantizer to prevent
+            # adding an additional Dequantizer.
+            if (is_quantizable(inbound.inbound_layers) and
+                    not isinstance(inbound.inbound_layers, Dequantizer)):
+                # Inbound will be quantized, add a Dequantizer after it and return the model
+                inbound_name = inbound.inbound_layers.name
+                model = insert_layer(model, inbound_name,
+                                        Dequantizer(name=f"{inbound_name}_float_output"))
+                warnings.warn(f"'{layer.name}' of type {layer.__class__} is not supported to "
+                               "quantize, a Dequantizer is added before it and quantization will "
+                               "stop at this layer.")
+                return model
+    return model
 
 
 def quantize(model, q_config, add_dequantizer=True):
@@ -91,7 +135,11 @@ def quantize(model, q_config, add_dequantizer=True):
     q_config_regex = {re.compile("/" + "(?:|_[\d]+)/".join(key[1:].split('/')) + "(?:|_[\d]+)$"):
                       (key[1:], obj) for key, obj in q_config.items()}
 
+    # Layers will no longer be quantized as soon as a Dequantizer is found
+    quantization_stopped = False
+
     def _replace_layer(layer):
+        nonlocal quantization_stopped
         config = layer.get_config()
 
         # Function to handle unsupported arguments in config
@@ -111,6 +159,7 @@ def quantize(model, q_config, add_dequantizer=True):
                 - second, remove unsupported arguments,
                 - then, we return the quantized layer with config updated
             """
+            nonlocal quantization_stopped
             class_name = layer.__class__.__name__
             # 1.1 Check if qlayer exists in custom layers
             if class_name in CUSTOM_QLAYERS:
@@ -118,8 +167,13 @@ def quantize(model, q_config, add_dequantizer=True):
             # 1.2 Else, return the float version of the layer
             else:
                 qlayer = layer
-                warnings.warn(
-                    f"'{class_name}' is not supported to quantize. It will be ignored.")
+                qlayer_class = qlayer.__class__
+                if not (qlayer_class in CUSTOM_QLAYERS.values() or
+                        qlayer_class in NO_FLOAT_CUSTOM_QLAYERS):
+                    warnings.warn(
+                        f"'{class_name}' is not supported to quantize. It will be ignored.")
+                # If a Dequantizer is found, quantization must be stopped
+                quantization_stopped = isinstance(layer, Dequantizer)
 
             # 2. Remove unsupported arguments
             pop_unsupported_args(qlayer)
@@ -129,6 +183,10 @@ def quantize(model, q_config, add_dequantizer=True):
                 config['quant_config'] = quantize_config
             return qlayer.from_config(config)
 
+        # When a not quantizable layer is found, stop quantization returning initial layer
+        if quantization_stopped:
+            return layer.from_config(config)
+
         # Replace float layer by quantized layer in three different ways:
         # 1. Check if layer is in quant_config (to quantize specific layers)
         layer_name = layer.name if layer.name[0] == "/" else "/" + layer.name
@@ -190,11 +248,13 @@ def quantize(model, q_config, add_dequantizer=True):
         # If no match, return the qlayer with config unchanged
         return get_quantize_layer(layer)
 
+    model = _handle_not_quantizable_layers(model)
     new_model = deep_clone_model(model, clone_function=_replace_layer)
     out = new_model.outputs
 
     # Append Dequantizer at the end of the model to convert the output to float value
-    if add_dequantizer and not isinstance(new_model.layers[-1], Dequantizer):
+    if (add_dequantizer and not isinstance(new_model.layers[-1], Dequantizer)
+         and not quantization_stopped):
         out = Dequantizer(name="float_output")(out)
     return Model(new_model.input, out)
 
