@@ -16,8 +16,8 @@
 # ******************************************************************************
 import tensorflow as tf
 
-from .qtensor import QTensor, floor_through
-
+from .qtensor import QTensor, round_through, floor_through
+from ..debugging import assert_equal, assert_less_equal
 
 class FixedPoint(QTensor):
     """A Tensor of integer values representing fixed-point numbers
@@ -36,8 +36,8 @@ class FixedPoint(QTensor):
             self.frac_bits = tf.cast(frac_bits, tf.float32)
         else:
             self.frac_bits = tf.convert_to_tensor(frac_bits, tf.float32)
-        tf.debugging.assert_less_equal(self.frac_bits, tf.convert_to_tensor(value_bits, tf.float32),
-                                       "Fractional bits cannot exceed value bits")
+        assert_less_equal(self.frac_bits, tf.convert_to_tensor(value_bits, tf.float32),
+            "Fractional bits cannot exceed value bits")
         self.value_bits = value_bits
         # We store integer values in a float tensor to speed up calculations
         if isinstance(values, tf.Tensor):
@@ -48,6 +48,10 @@ class FixedPoint(QTensor):
         self.values = QTensor.clamp(values, value_bits)
         self.shape = self.values.shape
 
+    @property
+    def per_tensor(self):
+        return tf.rank(self.frac_bits) == 0
+
     @staticmethod
     def quantize(x, frac_bits, value_bits):
         """Converts a float Tensor to a FixedPoint
@@ -70,7 +74,7 @@ class FixedPoint(QTensor):
             a :obj:`FixedPoint`
         """
         # Project float into integer fixed-point space
-        values = floor_through(x * FixedPoint._pow2(frac_bits))
+        values = round_through(x * FixedPoint._pow2(frac_bits))
         return FixedPoint(values, frac_bits, value_bits)
 
     @property
@@ -84,7 +88,7 @@ class FixedPoint(QTensor):
 
     @staticmethod
     def _pow2(n):
-        return tf.math.pow(2., tf.cast(n, tf.float32))
+        return tf.cast(2.0 ** tf.cast(n, tf.float64), tf.float32)
 
     def to_float(self):
         return self.values / FixedPoint._pow2(self.frac_bits)
@@ -104,7 +108,7 @@ class FixedPoint(QTensor):
             raise ValueError("Invalid bitwidth")
         if bits < self.value_bits:
             raise ValueError(
-                "Reducing the number of value bits is lossy: use a quantizer instead")
+               f"Cannot reduce value bits from {self.value_bits} to {bits}: use a quantizer instead")
         return FixedPoint(self.values, self.frac_bits, bits)
 
     def align(self, other=None):
@@ -135,6 +139,37 @@ class FixedPoint(QTensor):
         values = FixedPoint._lshift(self.values, shift)
         return FixedPoint(values, max_frac_bits, self.value_bits), shift
 
+    def downscale(self, frac_bits, value_bits):
+        """Encode a FixedPoint with a lower bitwidth
+
+        It applies a left/right shift to the source FixedPoint values to align
+        them on the target maximum fractional bits.
+
+        It then clips them to the boundaries specified by the target value bits.
+
+        Args:
+            frac_bits (:obj:`tensorflow.Tensor`): the target fractional bits
+            value_bits (int): the target value bits
+
+        Returns:
+            a :obj:`FixedPoint`
+        """
+        if value_bits > self.value_bits:
+            raise ValueError(
+               f"Cannot increase value bits from {self.value_bits} to {value_bits}:"
+                "use a promotion instead")
+        frac_bits = tf.cast(frac_bits, tf.float32)
+        # Evaluate the shift to apply to reach the target precision
+        shift = frac_bits - self.frac_bits
+        # The shift can be positive (left-shift) or negative (right-shift)
+        # In float it can be simulated by flooring after multiplying by the
+        # positive/negative shift PoT:
+        # - for positive shift, the results are integer and can safely be floored,
+        # - for negative shift, flooring simulates a right_shift.
+        values = floor_through(self.values * FixedPoint._pow2(shift))
+        # return a new FixedPoint with the target precision and bitwidth
+        return FixedPoint(values, frac_bits, value_bits)
+
     @staticmethod
     def _rshift(values, shift):
         return floor_through(values / FixedPoint._pow2(shift))
@@ -188,25 +223,33 @@ class FixedPoint(QTensor):
 
     def __add__(self, other):
         if isinstance(other, int):
-            # Convert integer into a 32-bit fixed-point with no fractional bits
-            return self + FixedPoint(other)
+            # Convert integer into a 32-bit fixed-point with no fractional bits,
+            # aligned with the current FixedPoint
+            return self + FixedPoint.quantize(other, self.frac_bits, value_bits=32)
         elif isinstance(other, FixedPoint):
-            s_frac_bits, s_int, o_values = self._align_values(other)
-            s_int += o_values
+            # Check that self and other are per-tensor
+            self.assert_per_tensor()
+            other.assert_per_tensor()
+            # Check that self and other are aligned
+            assert_equal(self.frac_bits, other.frac_bits)
             # Return a new FixedPoint
-            return FixedPoint(s_int, s_frac_bits, self.value_bits)
+            return FixedPoint(self.values + other.values, self.frac_bits, self.value_bits)
         raise TypeError(
             f"Unsupported operand type(s) for +: 'FixedPoint' and '{type(other)}'")
 
     def __sub__(self, other):
         if isinstance(other, int):
-            # Convert integer into a 32-bit fixed-point with no fractional bits
-            return self - FixedPoint(other)
+            # Convert integer into a 32-bit fixed-point with no fractional bits,
+            # aligned with the current FixedPoint
+            return self - FixedPoint.quantize(other, self.frac_bits, value_bits=32)
         elif isinstance(other, FixedPoint):
-            s_frac_bits, s_int, o_values = self._align_values(other)
-            s_int -= o_values
+            # Check that self and other are per-tensor
+            self.assert_per_tensor()
+            other.assert_per_tensor()
+            # Check that self and other are aligned
+            assert_equal(self.frac_bits, other.frac_bits)
             # Return a new FixedPoint
-            return FixedPoint(s_int, s_frac_bits, self.value_bits)
+            return FixedPoint(self.values - other.values, self.frac_bits, self.value_bits)
         raise TypeError(
             f"Unsupported operand type(s) for -: 'FixedPoint' and '{type(other)}'")
 
