@@ -38,6 +38,10 @@ class QFloat(QTensor):
         self.shape = self.values.shape
         self.value_bits = value_bits
 
+    @property
+    def per_tensor(self):
+        return tf.rank(self.scales.values) == 0
+
     @staticmethod
     def quantize(x, float_max, value_bits, scales_bits):
         """Converts a float Tensor to a QFloat
@@ -61,7 +65,7 @@ class QFloat(QTensor):
             :obj:`QFloat`: the QFloat representation.
         """
         # Evaluate QFloat scales
-        scales = float_max / QTensor.int_max(value_bits)
+        scales = tf.cast(float_max, tf.float32) / QTensor.int_max(value_bits)
         # Make sure the scales are at least equal to the scales resolution,
         # otherwise they would be quantized to zero
         scales_res = FixedPoint._pow2(-scales_bits)
