@@ -18,23 +18,34 @@ from copy import copy
 import tensorflow as tf
 
 
+from ..debugging import assert_equal
+
+
+@tf.function
+@tf.custom_gradient
 def round_through(x):
-    """Element-wise rounding to the closest integer with full gradient propagation.
-    A trick from [Sergey Ioffe](http://stackoverflow.com/a/36480182).
-
-    """
     rounded = tf.math.round(x)
-    return x + tf.stop_gradient(rounded - x)
+    def grad(upstream):
+        return upstream
+    return rounded, grad
 
 
+@tf.function
+@tf.custom_gradient
 def floor_through(x):
     floored = tf.math.floor(x)
-    return x + tf.stop_gradient(floored - x)
+    def grad(upstream):
+        return upstream
+    return floored, grad
 
 
+@tf.function
+@tf.custom_gradient
 def ceil_through(x):
     ceiled = tf.math.ceil(x)
-    return x + tf.stop_gradient(ceiled - x)
+    def grad(upstream):
+        return upstream
+    return ceiled, grad
 
 
 class QTensor(tf.experimental.ExtensionType):
@@ -52,6 +63,15 @@ class QTensor(tf.experimental.ExtensionType):
     value_bits: int = 7
     shape: tf.TensorShape  # Required to convert to a KerasTensor
 
+    @property
+    def per_tensor(self):
+        """Returns if QTensor is quantized per-tensor
+
+        Returns:
+            bool: True if QTensor is quantized per-tensor or False on per-axis case.
+        """
+        raise NotImplemented
+
     def to_float(self):
         """Returns a float representation of the QTensor
 
@@ -80,3 +100,8 @@ class QTensor(tf.experimental.ExtensionType):
     def clamp(values, value_bits):
         int_max = QTensor.int_max(value_bits)
         return tf.clip_by_value(values, -int_max, int_max)
+
+    def assert_per_tensor(self):
+        """Asserts that a QTensor is quantized per-tensor"""
+        name = self.__class__.__name__ if not hasattr(self.values, "name") else self.values.name
+        assert_equal(self.per_tensor, True, message=f"{name} is not per-tensor.")
