@@ -19,6 +19,7 @@ import tensorflow as tf
 import keras
 from typing import List, Union
 
+from ..debugging import assert_rank_at_least
 from .qtensor import QTensor, floor_through
 from .fixed_point import FixedPoint
 from .qfloat import QFloat
@@ -32,21 +33,6 @@ def warn_float_conversion(x, op):
     warnings.warn(f"Reverting {x_name} to float to perform {op} with Tensor")
 
 
-def assert_fp_per_tensor(x):
-    tf.debugging.assert_rank(x.frac_bits, tf.constant(0))
-
-
-def assert_qf_per_tensor(x):
-    tf.debugging.assert_rank(x.scales.values, tf.constant(0))
-
-
-def assert_per_tensor(x):
-    if isinstance(x, FixedPoint):
-        assert_fp_per_tensor(x)
-    elif isinstance(x, QFloat):
-        assert_qf_per_tensor(x)
-
-
 @tf.experimental.dispatch_for_binary_elementwise_apis(FixedPoint, tf.Tensor)
 def fp_tensor_binary_elementwise_api_handler(api_func, x, y):
     warn_float_conversion(x, api_func)
@@ -174,23 +160,19 @@ def fp_matmul(a: FixedPoint,
             f"Unsupported argument: output_type, provided {output_type}")
 
     # We don't support matmul between vectors
-    tf.debugging.assert_rank_at_least(a.values, 2)
-    tf.debugging.assert_rank_at_least(b.values, 2)
+    assert_rank_at_least(a.values, 2)
+    assert_rank_at_least(b.values, 2)
 
+    # Since the last dimension is collapsed by the matmul, (a,b) must be quantized per-tensor
     if not transpose_a:
-        # Since the last dimension is collapsed by the matmul, a must be quantized per-tensor
-        assert_fp_per_tensor(a)
+        a.assert_per_tensor()
+    if transpose_b:
+        b.assert_per_tensor()
 
     if isinstance(b, FixedPoint):
-        if transpose_b:
-            # Since the last dimension is collapsed by the matmul, b must be quantized per-tensor
-            assert_fp_per_tensor(b)
         b_frac_bits = b.frac_bits
     else:
         b_frac_bits = b.scales.frac_bits
-        if transpose_b:
-            # We only support tranposed matmul if the QFloat is per-tensor
-            assert_qf_per_tensor(b)
 
     # Do matmul on values
     m_values = floor_through(
@@ -211,14 +193,14 @@ def fp_shape(input: FixedPoint, out_type=tf.dtypes.int32, name=None):
 
 @tf.experimental.dispatch_for_api(tf.reshape)
 def fp_reshape(tensor: FixedPoint, shape, name=None):
-    assert_fp_per_tensor(tensor)
+    tensor.assert_per_tensor()
     output = tf.reshape(tensor.values, shape, name)
     return FixedPoint(output, tensor.frac_bits, tensor.value_bits)
 
 
 @tf.experimental.dispatch_for_api(tf.transpose)
 def fp_transpose(a: FixedPoint, perm=None, conjugate=False, name="transpose"):
-    assert_fp_per_tensor(a)
+    a.assert_per_tensor()
     output = tf.transpose(a.values, perm, conjugate, name)
     return FixedPoint(output, a.frac_bits, a.value_bits)
 
@@ -278,8 +260,8 @@ def fp_concat(values: List[FixedPoint], axis, name="concat"):
               ['we do not support concatenation over last axis. axis is:', axis])
 
     # FixedPoint tensors to concatenate should be per-tensor
-    assert_fp_per_tensor(values[0])
-    assert_fp_per_tensor(values[1])
+    values[0].assert_per_tensor()
+    values[1].assert_per_tensor()
 
     tf.assert_equal(values[0].frac_bits, values[1].frac_bits, message=f"the two\
             FixedPoint must have the same frac_bits. Receives {values[0].frac_bits}\
@@ -310,7 +292,7 @@ def fp_reduce_max(input_tensor: FixedPoint,
         :obj:`FixedPoint`: the maximum FixedPoint.
     """
     # We only support reduce_max if the input is per-tensor
-    assert_fp_per_tensor(input_tensor)
+    input_tensor.assert_per_tensor()
 
     # Reduce max
     s_values = tf.math.reduce_max(input_tensor.values,
@@ -339,7 +321,7 @@ def fp_gather(params: Union[FixedPoint, QFloat], indices, validate_indices=None,
     """
     # We do not support gather along the last axis if the QTensor is per-axis
     if axis in (-1, len(params.shape)):
-        assert_per_tensor(params)
+        params.assert_per_tensor()
     # Create a new QTensor, gather the indices in a desired axis
     x_gather = tf.gather(params.values, indices, validate_indices, axis, batch_dims, name)
     if isinstance(params, QFloat):
@@ -376,9 +358,7 @@ def fp_depthwise_conv2d(x: FixedPoint,
 
     # If x has multiple fractional bits, we must align the values along the last dimension before
     # the convolution collapses it
-    x = tf.cond(tf.rank(x.frac_bits) > 0,
-                lambda: x << (input_frac_bits - x.frac_bits),
-                lambda: x)
+    x = tf.cond(x.per_tensor, lambda: x, lambda: x << (input_frac_bits - x.frac_bits))
 
     if isinstance(depthwise_kernel, FixedPoint):
         filters_frac_bits = depthwise_kernel.frac_bits
@@ -428,9 +408,9 @@ def fp_convolution(
 
     # If a has multiple fractional bits, we must align the values along the
     # last dimension before the convolution collapses it
-    input = tf.cond(tf.rank(input.frac_bits) > 0,
-                    lambda: input << (input_frac_bits - input.frac_bits),
-                    lambda: input)
+    input = tf.cond(input.per_tensor,
+                    lambda: input,
+                    lambda: input << (input_frac_bits - input.frac_bits))
 
     if isinstance(filters, FixedPoint):
         filters_frac_bits = filters.frac_bits
